@@ -1,4 +1,4 @@
-// 	$Id: dicomInfo.cc,v 1.20 2006/10/04 18:27:57 valerio Exp $
+// 	$Id: dicomInfo.cc,v 1.25 2007/04/09 18:01:02 valerio Exp $
 
 //============================================================================
 // DCM is part of the CTN library, curtesy of
@@ -22,6 +22,10 @@
 //****************************************************************************
 
 #include "dicomInfo.h"
+#include <cstdlib>
+#include <iostream>
+
+using namespace std;
 
 //////////////////////////////////////////////////////////////////////////////
 ///
@@ -170,7 +174,7 @@ bool DICOMImage::MosaicAdjustments()
 bool DICOMImage::OpenFile(const char *path)
 {
 	bool retValue = true;
-	unsigned long options = DCM_ORDERLITTLEENDIAN | DCM_FORMATCONVERSION;
+	unsigned long options = DCM_ORDERLITTLEENDIAN | DCM_FORMATCONVERSION | DCM_VRMASK;
 	
 	// Try opening it "old style", if it fails because it's PART10, do it that way
 	
@@ -1455,10 +1459,10 @@ bool DICOMImage::ReadSlice(U16 *data, int slice)
 
 		// We can now copy the data from the correct spot
 		
-		int mosInd = mosCol * columns_ + mosRow * mosPix * rows_;
+		int mosInd = mosCol * columns_ + mosPix * mosRow * rows_;
 		int datInd = 0;
-		for (int col = 0; col < columns_; ++col, mosInd += mosInc)
-			for (int row = 0; row < rows_; ++row, ++mosInd, ++datInd)
+		for (int row = 0; row < rows_; ++row, mosInd += mosInc)
+			for (int col = 0; col < columns_; ++col, ++mosInd, ++datInd)
 				data[datInd] = volumeData_[mosInd];
 	}
 	else
@@ -1495,7 +1499,7 @@ bool DICOMImage::ReadVolume()
 	}		
 
 	DCM_ELEMENT element;
-	if ( (DCM_GetElement(&handle_, DCM_PXLPIXELDATA, &element) != DCM_NORMAL ) || (element.length > 4294967294) )
+	if ( (DCM_GetElement(&handle_, DCM_PXLPIXELDATA, &element) != DCM_NORMAL ) || (element.length > 0xFFFFFFFE) )
 	{
 		error_ = true;
 		string msg = string("DICOMImage::ReadVolumes: Error reading file: ") + imgPath_;
