@@ -1,6 +1,6 @@
-//  $Id: dinifti.cc,v 1.22 2006/10/04 18:58:29 valerio Exp $
+//  $Id: dinifti.cc,v 1.31 2007/05/22 14:39:25 valerio Exp $
 
-#define VERSION "2.22"
+#define VERSION "2.27"
 
 //****************************************************************************
 //
@@ -18,9 +18,13 @@
 //
 //****************************************************************************
 
+#include <dirent.h>
+#include <sstream>
+#include <algorithm>
 #include "dinifti.h"
 #include "niftiout.h"
-#include "dicomInfo.h"
+
+
 
 //****************************************************************************
 //
@@ -120,7 +124,8 @@ void ShowHelp(char **argv)
 		 << "                      'n1' - NIfTI-1 Single file **Default**" << endl 
 		 << " -d                   append series description to output file name(s)" << endl
 		 << " -s #                 number of slices per volume" << endl
-		 << " -v --verbose    enable verbose status output" << endl << endl
+		 << " -v --verbose    enable verbose status output" << endl
+		 << " -n --noact      do all processing, but do not write files" << endl << endl
 		 << " -h --help       print this help and exit" << endl
 		 << " -V --version    print version number and exit" << endl << endl
 		 << "I/O Options" << endl
@@ -195,6 +200,10 @@ int main(int argc, char **argv)
 		{
 			opts.verbose = true;
 		}
+		else if ( (strcmp(argv[parseArg], "-n") == 0) || (strcmp(argv[parseArg], "--noact") == 0) )
+		{
+			opts.noact = true;
+		}
 		else if (! strcmp(argv[parseArg], "-s") )
 		{
 			++parseArg;
@@ -282,7 +291,7 @@ int main(int argc, char **argv)
 //****************************************************************************
 
 void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *outName,
-					 DiniftiOptions opts)
+					 const DiniftiOptions& opts)
 {
 	// Figure out if the outName is a directory or the name of a file
 
@@ -396,7 +405,11 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 	
 	// Go through the series and see if any are a special case
 
-	for (SERIESMAP::iterator series = seriesMap.begin(); series != seriesMap.end(); series++)
+	// use this list to store series outNames to detect possible duplicates
+	list<string> series_ids;
+
+	// do not increment the iterator here as this has to be handled conditionally at the end
+	for (SERIESMAP::iterator series = seriesMap.begin(); series != seriesMap.end(); )
 	{
 		// For each series, check the number of groups (first image in first
 		// time point is sufficient)
@@ -414,6 +427,33 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 			cout << "Found new image series. Using output destination '" << si.outName << "'." << endl;
 		}
 
+		// check for duplicate series outNames
+		int dupes = count( series_ids.begin(), series_ids.end(), si.outName );
+
+		// if duplicate series name
+		if (dupes)
+		{
+			// store the old for a useful warning message
+			string old = si.outName;
+
+			// appen '+' until we have a unique name
+			while (dupes)
+			{
+				si.outName.append("+");
+				dupes = count( series_ids.begin(), series_ids.end(), si.outName );
+			}
+			// issue warning to notify the name change
+			cerr << endl << "\t^G^G**** WARNING: Duplicate image series name '" <<
+				old << "'" << endl
+				<< "\t**** New image series name is '" << si.outName << "'." << endl;
+
+			// important: reassign the modified series info (lost otherwise)
+			series->second = si;
+		}
+
+		// store the current image series name
+		series_ids.push_back(si.outName);
+
 		if ( (opts.numSlices == 0) && (si.numSlices == 1) && (si.maxAcqNum == 1) && (si.maxImgNum > si.minImgNum) )
 		{
 			cerr << endl << "\t**** StoreImageFiles: Warning: number of slices per volume = 1" << endl
@@ -471,8 +511,13 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 
 			// Now we can get rid of the old entries
 
-			seriesMap.erase(origSeriesID);
+			// FIRST increment the iterator THEN erase the entry
+			// otherwise the iterator becomes invalid
+			seriesMap.erase(series++);
 		}
+		else
+			// if not incremented above do here
+			++series;
 	}
 
 	// Move the added series to permanent storage
@@ -542,7 +587,7 @@ void StoreImageFiles(vector<string> &dicomFiles, SERIESMAP &seriesMap, char *out
 //
 //****************************************************************************
 
-void ProcessSeries(SeriesInfo &seriesInfo, DiniftiOptions opts)
+void ProcessSeries(SeriesInfo &seriesInfo, const DiniftiOptions& opts)
 {
 	// Create NIfTI header with default values
 	
@@ -585,21 +630,31 @@ void ProcessSeries(SeriesInfo &seriesInfo, DiniftiOptions opts)
 	
 	if ( (opts.niftiFormat == ANALYZE) || (opts.niftiFormat == NIfTI_DUAL) )
 		SetFileName(niftiHdr.iname, seriesInfo.outName.c_str(), opts.niftiFormat, opts.compressed, false);
-	
-	znzFile outfp = NULL;
-	outfp = nifti_image_write_hdr_img(&niftiHdr, 2, "wb");
-	
-	if (outfp == NULL)
+
+	if (!opts.noact)
 	{
-		exit(1);
-	}
-			
-	if (! NIfTIWriteData(outfp, niftiHdr, seriesInfo.imageMap) )
-	{
-		cerr << "Writing output file failed, aborting." << endl << endl;
+		znzFile outfp = NULL;
+		outfp = nifti_image_write_hdr_img(&niftiHdr, 2, "wb");
+		
+		if (outfp == NULL)
+		{
+			exit(1);
+		}
+				
+		if (! NIfTIWriteData(outfp, niftiHdr, seriesInfo.imageMap) )
+		{
+			cerr << "Writing output file failed, aborting." << endl << endl;
+			znzclose(outfp);
+			exit(1);
+		}
+
 		znzclose(outfp);
-		exit(1);
 	}
 
-	znzclose(outfp);
+	// Clean up time
+
+	delete [] niftiHdr.fname;
+	if (niftiHdr.iname != NULL) delete [] niftiHdr.iname;
+
+
 }
